package com.yihu.jw.util.sql;


import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.persistence.Column;
import org.hibernate.transform.ResultTransformer;

public class BeanTransformer<T> implements ResultTransformer {
    private final Class<T> mapClass;
    public Map<String, String> columnToProperty;
    public Map<String, String> propertyToColumn;

    public BeanTransformer(Class<T> mapClass) {
        this.mapClass = mapClass;
        this.initColumnsToProperties();
        this.initPropertiesToColumns();
    }

    public Object transformTuple(Object[] tuple, String[] aliases) {
        try {
            PropertyDescriptor[] props = this.propertyDescriptors();
            int[] col2Pro = this.mapColumnsToProperties(aliases, props);
            return this.createBean(tuple, this.mapClass, props, col2Pro);
        } catch (Exception var5) {
            var5.printStackTrace();
            return null;
        }
    }

    public List transformList(List list) {
        return list;
    }

    protected T newInstance(Class<T> c) throws Exception {
        try {
            return c.newInstance();
        } catch (IllegalAccessException | InstantiationException var3) {
            throw new Exception("Cannot create " + c.getName() + ": " + var3.getMessage());
        }
    }

    protected int[] mapColumnsToProperties(String[] aliases, PropertyDescriptor[] props) throws SQLException {
        int[] col2Pro = new int[aliases.length];
        Arrays.fill(col2Pro, -1);

        for(int col = 0; col < aliases.length; ++col) {
            String columnName = aliases[col];
            String propertyName = (String)this.columnToProperty.get(columnName.toUpperCase());

            for(int i = 0; i < props.length; ++i) {
                if (propertyName != null & props[i] != null && propertyName.equals(props[i].getName())) {
                    col2Pro[col] = i;
                    break;
                }
            }
        }

        return col2Pro;
    }

    private T createBean(Object[] tuple, Class<T> type, PropertyDescriptor[] props, int[] col2Pro) throws Exception {
        T bean = this.newInstance(type);

        for(int i = 0; i < col2Pro.length; ++i) {
            int realIndex = col2Pro[i];
            if (realIndex != -1) {
                Object value = tuple[i];
                PropertyDescriptor prop = props[realIndex];
                this.callSetter(bean, prop, value);
            }
        }

        return bean;
    }

    private void callSetter(Object target, PropertyDescriptor prop, Object value) throws Exception {
        Method setter = prop.getWriteMethod();
        if (setter != null) {
            Class[] params = setter.getParameterTypes();

            try {
                if (value instanceof Date) {
                    String targetType = params[0].getName();
                    if ("java.sql.Date".equals(targetType)) {
                        value = new java.sql.Date(((Date)value).getTime());
                    } else if ("java.sql.Time".equals(targetType)) {
                        value = new Time(((Date)value).getTime());
                    } else if ("java.sql.Timestamp".equals(targetType)) {
                        Timestamp tsValue = (Timestamp)value;
                        int nanos = tsValue.getNanos();
                        value = new Timestamp(tsValue.getTime());
                        ((Timestamp)value).setNanos(nanos);
                    }
                }

                if (this.isCompatibleType(value, params[0])) {
                    setter.invoke(target, value);
                } else if (params[0].equals(Long.TYPE) && value instanceof Integer) {
                    setter.invoke(target, value);
                } else {
                    if (!params[0].equals(Long.class) || !(value instanceof Integer)) {
                        throw new Exception("Cannot set " + prop.getName() + ": incompatible types, cannot convert " + value.getClass().getName() + " to " + params[0].getName());
                    }

                    setter.invoke(target, Long.valueOf(value.toString()));
                }

            } catch (IllegalAccessException | InvocationTargetException | IllegalArgumentException var9) {
                throw new SQLException("Cannot set " + prop.getName() + ": " + var9.getMessage());
            }
        }
    }

    private void initColumnsToProperties() {
        this.columnToProperty = new HashMap();
        Field[] fields = this.mapClass.getDeclaredFields();
        Field[] var2 = fields;
        int var3 = fields.length;

        for(int var4 = 0; var4 < var3; ++var4) {
            Field f = var2[var4];
            Column column = (Column)f.getAnnotation(Column.class);
            if (column != null) {
                String columnName = column.name();
                String propertyName = f.getName();
                this.columnToProperty.put(columnName.toUpperCase(), propertyName);
            }
        }

    }

    private void initPropertiesToColumns() {
        this.propertyToColumn = new HashMap();
        Field[] fields = this.mapClass.getDeclaredFields();
        Field[] var2 = fields;
        int var3 = fields.length;

        for(int var4 = 0; var4 < var3; ++var4) {
            Field f = var2[var4];
            Column column = (Column)f.getAnnotation(Column.class);
            if (column != null) {
                String columnName = column.name();
                String propertyName = f.getName();
                this.propertyToColumn.put(propertyName, columnName.toUpperCase());
            }
        }

    }

    private PropertyDescriptor[] propertyDescriptors() throws Exception {
        BeanInfo beanInfo = null;

        try {
            beanInfo = Introspector.getBeanInfo(this.mapClass);
        } catch (IntrospectionException var3) {
            throw new Exception("Bean introspection failed: " + var3.getMessage());
        }

        return beanInfo.getPropertyDescriptors();
    }

    private boolean isCompatibleType(Object value, Class<?> type) {
        if (value != null && !type.isInstance(value)) {
            if (type.equals(Integer.TYPE) && value instanceof Integer) {
                return true;
            } else if (type.equals(Integer.TYPE) && value instanceof Long) {
                return true;
            } else if (type.equals(Long.TYPE) && value instanceof Long) {
                return true;
            } else if (type.equals(Double.TYPE) && value instanceof Double) {
                return true;
            } else if (type.equals(Float.TYPE) && value instanceof Float) {
                return true;
            } else if (type.equals(Short.TYPE) && value instanceof Short) {
                return true;
            } else if (type.equals(Byte.TYPE) && value instanceof Byte) {
                return true;
            } else if (type.equals(Character.TYPE) && value instanceof Character) {
                return true;
            } else {
                return type.equals(Boolean.TYPE) && value instanceof Boolean;
            }
        } else {
            return true;
        }
    }
}
